/*
30-June-2005  Drew Baden
1. Start with s2 version of the sandwich self-triggering board....
2. Modify so that each Xilinx in the HTR sends, at 80MHz
    o 2 bits per channel (24 channels, 48 bits)
	 o 16 bits for sumET
	 o 1 bit for "frame"
	 o 1 bit for L1A
  Total: 66 bits @ 80MHz
3. We need some output pins to debug, so we'll use the "Wesley" connectors:
  J4 and J5 are the 50pin connectors that are available and look like this
  on the board:
  
     J4          J5         J7
  -50- -25-   -50- -25-    |||||
  -49- -24-   -49- -24-    |||||
  
  -27- -02-   -27- -02-    |||||
  -26- -01-   -26- -01-    |||||
                            J6
on J4 and J5:  pin 1 is the bottom right pin in the 25row by 2 columns

On J4, there are 23 free pins
On J5, there are 23 free pins

Pins 49 and 50 on J4 are REFCLK+ and REFCLK- driven from the FPGA
on B12.  The firmware below drives this pin (B12) from the DCM through
a TTL->LVPECL converter (U2A).  The origin of this clock is the 40MHz
clock from the TTCrx

We need an 80MHz clock to latch things.  The DCM produces a 80MHz clock
  which is 2x the TTC clock.  Drive this onto J5/pin1 from U3 (CK80)

Data from each HTR Xilinx:

*/  

`include "define.v"
module SandwichTop 
 ( 
 input  CLKp, CLKn, // 40MHz differential RX_CLK
 input  BCOp, BCOn, // RX_BC0 common to all SLB boards
 input  SLB_ID,
 input  TTC_CK40Des1, TTC_BRCST_STRB1, //TTC_BCR,
 input  SCLK,       // 80 MHz Clock from onboard crystal
 input  LOC_nCS,  LocWRITE_b, 
 input  [7:2] TTC_BRCST,
 input [23:0] TOP_D, BOT_D,		// data from HTR motherboard
 input [11:0] TOPbus, BOTbus,    //               "
 input  [`LB_Addr_Size-1:0]  LocAdd,  // The Address bus.
 inout  [`REG_WIDTH-1:0]     LocBus,  // The bidir Data bus 
 output SLB_CFGDONE,
 output [22:0] SPY4,		// ouput onto J4  see http://www.physics.umd.edu/hep/HTR/luminosity/uwspy.html
 output [22:0] SPY5,		// ouput onto J5                            ditto

 output LVDS1n, LVDS1p, LVDS2n, LVDS2p,// to MDR14 J10
 output REFCLK, TRSTN,  LED_CONTROL,
 input [3:0] PH_RxD,
 input PH_RxDV,
 input PH_RxER,
 input PH_RxCLK,
 output [3:0] PH_TxD,
 output PH_TxER,
 output PH_TxEN,
 input PH_TxCLK,
 input PH_COL,
 input PH_CRS,
 output PH_MDC,
 inout PH_MDIO,
 output PH_MDIS,
 output [1:0] PH_SLEW,
 output PH_CLK,
 output PH_RESET,
 output PH_PAUSE,
 output PH_SLEEP,
 output PH_PDWN

 
 );


//
// declarations...
//
reg[26:0] clk_scaled;
wire  VME_Hard_rst, VME_Soft_rst, VME_Start, VME_Stop, VME_LVDSout,
       run_status, BunchEr, GateTTCcommands, Hard_rst, Soft_rst;
wire resetn = ~Hard_rst;
wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
             TTC_test4, TTC_test5, TTC_test6, TTC_test7,
             TTC_test8, TTC_test9, TTC_testA, TTC_testB,
             TTC_testC, TTC_testD, TTC_testE, TTC_testF;

wire [8:1]  TPSpyFifoEmpty, TPSpyFifoFull, TPSpyFifoRd,
             InstantCntErr,  InstantR1sErr;
wire [15:0] AccCntErr, AccR1sErr;  
wire [35:0]  TPSpyFIFO1_out, TPSpyFIFO2_out, TPSpyFIFO3_out;
reg VME_LVDSout_D, LVDSout1, LVDSout2;
wire [7:0] LastTTCcommand;
reg [23:0] rTOP_D, rBOT_D;
reg [11:0] rTOPbus, rBOTbus;
reg [15:0] RxClkCnt, RxBC0Cnt, TClkCnt;
wire[7:0] mux_out;
wire[1:0] ch1,ch2;
wire[3:0] mch1;
wire[5:0] sum0, sum1, sum2, sum3;
wire hist_localbus_read;
wire[11:0] hist_localbus_bin;
wire hist_zero;
wire[11:0] bucket;  // reset when "BC0" is asserted
wire goRead;
wire [4:0] hist_localbus_select;
wire HistogramMode;
wire[2:0] lumState;
wire latch_all, hdone;
wire LEDdoBC0;
reg[11:0] count_bucket, latch_bucket;
wire[7:0] TTC_BC0_pattern;
wire TTC_BC0_seen;
//
// for ethernet stuff...
//
   wire packet_busy;
	wire packet_clk;
	wire [47:0] DestMAC; // destination MAC address
	wire [47:0] HLXMAC; // transmitter MAC address 
	wire [7:0] hlxId; // HLX id byte
	reg [10:0] packet_len; // packet data size
	wire [31:0] firstOrbit,lastOrbit;
	reg [5:0] packet_sel;
	reg packet_send;
	wire histogram_read_ack;
	wire histogram_read_strobe;
	wire [31:0] histogram_output_read;
	wire [15:0] histogram_output_addr;
	wire [15:0] vme_pkt_read,vme_pkt_write;
	wire [8:0] vme_pkt_addr;
	wire vme_pkt_we;
	reg [2:0] packet_next;
   wire ethTransmit, ethPacketReset;
   reg ethPacketSent;
	reg[11:0] ethCounter;
   wire [4:0] ethHistSel;
	reg doingEthernet;
	wire xmit_busy;
	wire alldone;
	reg[1:0] eState;
	wire[2:0] slave_state;
	wire[10:0] ethLength;
   wire ethLenOver;


//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

wire   RxClk_Buf, SCLK_buf;
wire   Tclk0, Tclk180, TTC_CK, Tclk2x, Tclk2x180, TTC_CK2x, TTC_DLL_lock;
reg    [2:0] UnlockCnt; // counter for proper reset of DLL 
//
// Instantiate the Differential Global Input Buffer 
//
IBUFGDS_LVPECL_25 difclkbuf (.I(CLKp), .IB(CLKn), .O(RxClk_Buf));
IBUFG sclkbuf ( .I(SCLK), .O(SCLK_buf) );
//                  
// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
//
  FDS flop1 (.D(1'b0), .C(SCLK_buf), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SCLK_buf), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SCLK_buf), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SCLK_buf), .Q(out4)); 
assign ConfigRst = (out2 | out3 | out4); 
//
// instantiate the DCM
//
DCM TTC_DCM
   (.CLKIN(TTC_CK40Des1), .CLKFB(TTC_CK),  .RST(ConfigRst),  
    .CLK0(Tclk0),   .CLK90(),  .CLK180(Tclk180),  .CLK270(),	
    .CLK2X(Tclk2x),  .CLK2X180(Tclk2x180),
    .CLKDV(),   .CLKFX(),   .CLKFX180(),
    .LOCKED(TTC_DLL_lock),  .STATUS(),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

BUFG TCK40BUF (.I(Tclk0), .O(TTC_CK));
BUFG TCK40BUF180 (.I(Tclk180), .O(TTC_CK180));  
BUFG TCK2XBUF (.I(Tclk2x), .O(TTC_CK2x180)); 
BUFG TCK2XBUF180 (.I(Tclk2x180), .O(TTC_CK2x));
   assign packet_clk=TTC_CK180;

//
// register data since it is arriving at 80MHz. note that in the HTR
// firmware, a 36-bit register called "outbit" is sent out at 80MHz.
// here is the correspondence between outbit and TOP_D and TOPbus
// (same for outbit and BOT_D and BOTbus)
//
// outbit[23:0] = MSB/LSB of 2-level for each of 24 channels
// outbit[31:24]= sumET.  sumET is a 15-bit number, so one of the
//                80MHz clocks will deliver sumET[7:0] and the
//                other will deliver {1,sumET[14:8]}.  which one?
//                the same "frame" for when the LSB of the 2-bit
//                level and the lower 8 bits of sumET, and the
//                same frame for the MSB of the 2-bit level and
//                the upper 7 bits of sumET
// outbit[32]   = 80MHz clock from the HTR (clk2x)
// outbit[33]   = 40MHz clock from the HTR (clk)
// outbit[34]   = TTC_TestEnable (or BC0 in future versions)
// outbit[35]   = frame (should be equivalent to outbit[33]
//
// TOPbus[11:0] = outbit[35:24]
// TOP_D[23:0]  = outbit[23:0]
//
always @ (posedge TTC_CK2x) begin 
  rTOP_D <= TOP_D;
  rBOT_D <= BOT_D;
  rTOPbus <= TOPbus;
  rBOTbus <= BOTbus;
  end
//
// now form the sums each channel
//
wire [7:0] sdebug;
assign mch1 = sdebug[7:4];
assign ch1 = sdebug[3:2];
assign ch2 = sdebug[1:0];
wire rframe = rTOPbus[11];  
wire rTTC_TestEnable = rTOPbus[10];
wire htrCLK = rTOPbus[9];
wire htrCLK2x = rTOPbus[8];
wire[7:0] etsum_partial = rTOPbus[7:0];		  
wire[14:0] etSUM;
//
// this module changes from 80MHz to 40MHz, so it has to put the 2 12.5ns cycles
// together
//
channelSums TOPsum(.clk(TTC_CK180),.clk2x(TTC_CK2x),.resetn(resetn),
	.frame(rframe),.data(rTOP_D),
	.etsum_partial(etsum_partial),.etSUM(etSUM),
	.sum0(sum0),.sum1(sum1),.sum2(sum2),.sum3(sum3),
	.debug(sdebug));
//
// etSUM is 15 bits, but the histograms are 18 bits and use 6-bit inputs.  so just
// for this project, take the 15 bit sums and turn them into 6-bit sums by chopping
// off the bottom 6 bits and saturating the high end.
//
wire[5:0]etSUMhist = etSUM[14:12] > 0 ? 'h3F : etSUM[11:6];
//
// use TTC_BRCST_STRB1 as our BC0...for now...
//
wire bc0_strobe = TTC_BC0_seen;
//wire bc0_strobe = TTC_BRCST_STRB1;
//
// histograms go here...to start filling, issue the "START" command and set the CSR
// bit to enable histogram filling. filling will commence after the next "BC0"
//
// the histogram to BRAM mapping goes like this:
//
//      histogram bin     RAM    address
//             0           0        0
//             1           1        0
//             2           2        0
//             3           3        0
//             4           0        1
//             5           1        1
//             6           2        1
//             7           3        1
//             8           0        2
//             9           1        2
//            10           2        2
//            11           3        2
// and so on up to:
//          3560           0      891
//          3561				1      891
//          3562 				2      891
//          3563 				3      891
//
// note:  the mapping is given by:
//        if bin[11:0], then RAM=bin[1:0] and address=bin[11:2]
//
wire startHistograms = HistogramMode && run_status;
wire [15:0] rv1,rv2,rv3,rv4;
wire [1:0] rs1,rs2,rs3,rs4;
wire rw1,rw2,rw3,rw4;
wire [3:0] sel0,sel1,sel2,sel3;
wire[9:0] addr0,addr1;
wire[1:0] hstate0,hstate1;
wire[4:0] hLatch;
wire[15:0] latch_h0, latch_h1,latch_h2,latch_h3,latch_sum;
wire[11:0] freeBucket;
wire szero;
wire[31:0] ethData;
wire readEnable = doingEthernet ? histogram_read_strobe : hist_localbus_read;
wire [9:0] ethAddr;
wire ethRAM;
wire [11:0] maxOrbit = 'd1023;  // actually, the maximum is 1 more than this!
//
// when we read via ethernet, let's double the clock from 40 to 80MHz so that
// we don't starve the MAC core.  we could also try to eliminate states but
// this will work...
//
wire hclk_mux = doingEthernet ? Tclk2x : TTC_CK180;
wire hclk180_mux = doingEthernet ? Tclk2x180 : TTC_CK;
BUFG TCK2XBUF_mux (.I(hclk_mux), .O(HCLK)); 
BUFG TCK2XBUF180_mux (.I(hclk180_mux), .O(HCLKn)); 
wire histclk, histnclk;
histLUM HFLUM(
	// inputs:
	.clk(HCLKn),.clkn(HCLK),.resetn(resetn),
//	.clk(TTC_CK180),.clkn(TTC_CK),.resetn(resetn),
	.clkout(histclk),.clknout(histnclk),
	.fEnable(startHistograms),.BC0(bc0_strobe),.zero(hist_zero),
	.rEnable(readEnable),.rAddress(hist_localbus_bin),.readWhich(hist_localbus_select),
	.sum0(sum0),.sum1(sum1),.sum2(sum2),.sum3(sum3),.etsum(etSUMhist),.maxOrbit(maxOrbit),
	// outputs:
	.goRead(goRead),.bucket(bucket),.histDone(HistogramsDone),.latch_all(latch_all),.latchEach(hLatch),
	.hData0(latch_h0),.hData1(latch_h1),.hData2(latch_h2),.hData3(latch_h3),.sData(latch_sum),
	.orbitDone(hdone),
	.enable_ram_write(enable_ram_write),.enable_ram_read(enable_ram_read),.state(lumState),
	// ethernet
	.ethData(ethData),.ethAddr(ethAddr),.ethRAM(ethRAM),.doingEthernet(doingEthernet),
	.ethWhich(ethHistSel),
	// debugging:
	.rv1(rv1),.rv2(rv2),.rv3(rv3),.rv4(rv4),
	.rs1(rs1),.rs2(rs2),.rs3(rs3),.rs4(rs4),
	.rw1(rw1),.rw2(rw2),.rw3(rw3),.rw4(rw4),
	.sel0(sel0),.sel1(sel1),.sel2(sel2),.sel3(sel3),
	.addrH0(addr0),.addrH1(addr1),.h0State(hstate0),.h1State(hstate1),.freeBucket(freeBucket),
	.szero(szero)
	);
//
// spy output onto wisconsin receiver
//	see http://www.physics.umd.edu/hep/HTR/luminosity/uwspy.html
// there are 23 pins available per wisconsin connectors J4 and J5
// SPY4 has is hooked up to various bits of J4, and likewise for SPY5 and J5
// i load up SPY4 and SPY5 in some kind of random fashion due to how I hook
// up the agilent.
//
//assign SPY4 = clk_scaled[22:0];
//assign SPY5 = clk_scaled[22:0];
reg[15:0] spyj5, spyj4;
//wire[15:0] spyj5, spyj4;
assign SPY4 = {5'h0,spyj4[15:12],2'h0,spyj4[11:0]};
assign SPY5 = {6'h0,spyj5[15:8],1'b0,spyj5[7:0]};
//assign spyj4 = {TTC_CK2x,TTC_CK180,bc0_strobe,startHistograms,HistogramsDone,
//                doingEthernet,histogram_read_strobe,hist_localbus_read};
//assign spyj5 = clk_scaled[15:0];
wire ethClk = PH_CLK;
wire[11:0] temp = {enable_ram_write,enable_ram_read,lumState[2:0],hist_localbus_read,
		                latch_all,bc0_strobe,hdone,startHistograms,szero,HistogramMode};
always @ (mux_out or ch1 or ch2 or mch1 or sum0 or sum1 or sum2 or sum3 or rTOP_D or TOP_D or bucket
  or TTC_CK180 or TTC_CK2x or TTC_BRCST_STRB1 or rframe) begin
	case (mux_out[3:0])
		0: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
			spyj5[11:0] = {ch2,ch1,mch1,TOP_D[1:0],rTOP_D[1:0]};
			spyj4 = {spyj5[15:12],sum1,sum0};
			end
		1: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = temp;
			spyj4 = {sum1[3:0],bucket};
			end
	   2: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = {ch2,ch1,mch1,TOP_D[1:0],rTOP_D[1:0]};
			spyj4 = {sum1[3:0],bucket};
			end
		3: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
			spyj5[11:0] = temp;
			spyj4 = {mch1,freeBucket};
//			spyj4 = {mch1,bucket};
////		   spyj4 = {sum0[3:0],bucket};
		   end
	   4: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = rTOP_D[11:0];
			spyj4 = {mch1,rTOP_D[23:12]};
			end
	   5: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = temp;
			spyj4 = {sel0[3:0],hstate0[1:0],addr0[9:0]};
			end
	   6: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = temp;
			spyj4 = {sel1[3:0],hstate1[1:0],addr1[9:0]};
			end
		7: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = temp;
		   spyj4 = {sel1[3:0],rw4,rw3,rw2,rw1,
				rs4[1:0],rs3[1:0],rs2[1:0],rs1[1:0]};
		   end
		8: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, startHistograms};
		   spyj5[11:0] = bucket;
		   spyj4 = rv1;
		   end
		9: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, startHistograms};
		   spyj5[11:0] = bucket;
		   spyj4 = rv2;
		   end
		'd10: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
			spyj5[11:0] = {ch2,ch1,mch1,TOP_D[1:0],rTOP_D[1:0]};
			spyj4 = {rframe,rTTC_TestEnable,htrCLK,htrCLK2x,etsum_partial};
		   end
		'd11: begin
//         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, startHistograms};
//		   spyj5[11:0] = bucket;
//		   spyj4 = rv4;
         spyj5[15:0] = histogram_output_addr;
			spyj4 = {histnclk,slave_state[2:0],doingEthernet,
						xmit_busy,eState[1:0],
						packet_next[2:0],packet_busy,
						packet_send,histogram_read_ack,histogram_read_strobe,histclk};
		   end
		'd12: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = {enable_ram_write,enable_ram_read,lumState[2:0],hist_localbus_read,
		                latch_all,goRead,hLatch[3:0]};
			spyj4 = histogram_output_addr;
		   end
		'd13: begin
			// use this to check that all of the outputs are okok
			spyj5[15:0] = clk_scaled[15:0];
			spyj4[15:0] = clk_scaled[15:0];
		   end
		'd14: begin
         spyj5[15:12] = {TTC_CK180, TTC_CK2x, TTC_BRCST_STRB1, rframe};
		   spyj5[11:0] = {enable_ram_write,enable_ram_read,lumState[2:0],hist_localbus_read,
		                latch_all,goRead,hLatch[3:0]};
			spyj4 = {ethTransmit,ethPacketReset,ethPacketSent,doingEthernet,
						xmit_busy,alldone,eState[1:0],
						packet_next[2:0],packet_busy,
						packet_send,histogram_read_ack,histogram_read_strobe,ethClk};
		   end
		'd15: begin
         spyj5[15:0] = histogram_output_addr;
			spyj4 = {ethTransmit,slave_state[2:0],doingEthernet,
						xmit_busy,eState[1:0],
						packet_next[2:0],packet_busy,
						packet_send,histogram_read_ack,histogram_read_strobe,ethClk};
		   end
   endcase
end
//
// bit 8 of the CR (0x0) is used to toggle the LED when we get a BC0.
//
reg[10:0] rsff;
always @ (posedge bc0_strobe)
  if (VME_Hard_rst) rsff <= 0;
  else rsff <= rsff + 1;
always @ (posedge TTC_CK2x) clk_scaled <= #1 clk_scaled + 1;
assign LED_CONTROL = LEDdoBC0 ? rsff[10] : clk_scaled[25];

//LED_Sine breathing(.clock(SCLK_buf), .nReset(1'b1), .led(LED_CONTROL));
 assign REFCLK = TTC_CK;
 assign SLB_CFGDONE = SLB_ID; // in case ECAL TCC uses this board
 assign TRSTN = 1'b1; // Wisc. Rec_Mezz JTAG reset.
 IBUFDS_LVPECL_25 RxBC0buf (.I(BCOp), .IB(BCOn), .O(RxBC0));
 OBUFDS_LVDS_25 pair1 (.I(LVDSout1), .O(LVDS1p), .OB(LVDS1n));
 OBUFDS_LVDS_25 pair2 (.I(LVDSout2), .O(LVDS2p), .OB(LVDS2n));
//
// decode the TTC to keep track (just for checking)
//
decodeTTC TTCdecode (
 .TTC_CK40Des1(TTC_CK),
 .reset(VME_Hard_rst), 
 .TTC_Brcst(TTC_BRCST),
 .TTC_BrcstStr(TTC_BRCST_STRB1),   
 .TTC_test0(TTC_test0), .TTC_test1(TTC_test1),
 .TTC_test2(TTC_test2), .TTC_test3(TTC_test3),
 .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
 .TTC_test6(TTC_test6), .TTC_test7(TTC_test7),
 .TTC_test8(TTC_test8), .TTC_test9(TTC_test9),
 .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
 .TTC_testC(TTC_testC), .TTC_testD(TTC_testD),
 .TTC_testE(TTC_testE), .TTC_testF(TTC_testF),
 .TTC_BC0_pattern(TTC_BC0_pattern),
 .TTC_BC0_seen(TTC_BC0_seen),
 .LastTTCcommand(LastTTCcommand)
 );

always @ (posedge TTC_CK) begin
  LVDSout1 <= VME_LVDSout;
  LVDSout2 <= VME_LVDSout;
  end

//
// put the entire 36 bits from each Xilinx (HTR TOP and BOT) into a fifo
// which is 512 deep and 36 bits wide (RAMB16_S36_S36).  however, the
// readout data path is only 16 bits wide.  what to do?   we'll have to 
// read it 16 bits at a time!	 therefore we'll need a different register
// for the 3 reads (3x16 > 36 bits).  all this will be done inside the
// localbusslave routine
//
fifo_ic_v2  TPSpyFIFO1 
 ( .fifo_gsr_in(Hard_rst),       .write_data_in({rTOPbus, rTOP_D}), 
   .write_clock_in(TTC_CK2x),        .write_enable_in(run_status),
   .read_clock_in(TTC_CK),           .read_enable_in(TPSpyFifoRd[1]),
   .read_data_out({TPSpyFIFO1_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[1]),      .empty_out(TPSpyFifoEmpty[1])  );

fifo_ic_v2  TPSpyFIFO2 
 ( .fifo_gsr_in(Hard_rst),       .write_data_in({rBOTbus, rBOT_D}), 
   .write_clock_in(TTC_CK2x),        .write_enable_in(run_status),
   .read_clock_in(TTC_CK),           .read_enable_in(TPSpyFifoRd[2]),
   .read_data_out({TPSpyFIFO2_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[2]),      .empty_out(TPSpyFifoEmpty[2])  );
												 
fifo_ic_v2  TPSpyFIFO3 
 ( .fifo_gsr_in(Hard_rst),       .write_data_in({freeBucket,sum3,sum2,sum1,sum0}), 
   .write_clock_in(TTC_CK),        .write_enable_in(run_status),
   .read_clock_in(TTC_CK),           .read_enable_in(TPSpyFifoRd[3]),
   .read_data_out({TPSpyFIFO3_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[3]),      .empty_out(TPSpyFifoEmpty[3])  );

//
// put counters to see if there are any non-zero values in the sums, count
// the number of buckets between the 1st non-zero value and bc0_strobe.
// this is for debugging!!!
//
wire[12:0] sum0_count;
wire count0_nonzero = (sum0 > 'h0);
wire[12:0] sum1_count;
wire count1_nonzero = (sum1 > 'h0);
wire[12:0] sum2_count;
wire count2_nonzero = (sum2 > 'h0);
wire[12:0] sum3_count;
wire count3_nonzero = (sum3 > 'h0);
count_buckets SUM0 (.clock(TTC_CK),.reset(VME_Hard_rst),.start(bc0_strobe),
	.stop(count0_nonzero), .re_enable(read_count), .count(sum0_count));
count_buckets SUM1 (.clock(TTC_CK),.reset(VME_Hard_rst),.start(bc0_strobe),
	.stop(count1_nonzero), .re_enable(read_count), .count(sum1_count));
count_buckets SUM2 (.clock(TTC_CK),.reset(VME_Hard_rst),.start(bc0_strobe),
	.stop(count2_nonzero), .re_enable(read_count), .count(sum2_count));
count_buckets SUM3 (.clock(TTC_CK),.reset(VME_Hard_rst),.start(bc0_strobe),
	.stop(count3_nonzero), .re_enable(read_count), .count(sum3_count));

////////////////////////////////////////////////////////
reg [11:0] nbsent;
LocBusSlave LocBusSlave (
    .clk(TTC_CK),      .rst(Hard_rst), 
    .LocAddr(LocAdd),    .LocData(LocBus), 
    .LocCS_b(LOC_nCS),   .LocWRITE_b(LocWRITE_b), 
    .LB_STATE(),   .Din(), .Dout(),      .Tri(), 
    .RxDLL_lock(1'b0), .TTC_DLL_lock(TTC_DLL_lock), .UnlockCnt(UnlockCnt),
    .DLL_status(16'b0),.LEDdoBC0(LEDdoBC0),
    .LastTTCcommand(LastTTCcommand), .GateTTCcommands(GateTTCcommands), 
    .VME_Hard_rst(VME_Hard_rst),     .VME_Soft_rst(VME_Soft_rst), 
    .VME_Start(VME_Start),           .VME_Stop(VME_Stop), 
    .VME_LVDSout(VME_LVDSout),    .run_status(run_status),
	 .packet_next(packet_next),.ethTransmit(ethTransmit), .ethHistSel(ethHistSel),
	 .ethPacketReset(ethPacketReset),.ethPacketSent(ethPacketSent),.ethCounter(nbsent),
	 .ethLength(ethLength),.ethLenOver(ethLenOver),
	 .read_count(read_count),.sum0_count(sum0_count),.sum1_count(sum1_count),
	 .sum2_count(sum2_count),.sum3_count(sum3_count), .DestMAC(DestMAC), .HLXMAC(HLXMAC),        
    .RxClkCnt(RxClkCnt), .RxBC0Cnt(RxBC0Cnt),
    .TClkCnt(TClkCnt), .TxBC0Cnt(16'b0), .TTC_BC0_pattern(TTC_BC0_pattern),
    .TPSpyFifoRd(TPSpyFifoRd),
    .TPSpyFifoEmpty(TPSpyFifoEmpty),   .TPSpyFifoFull(TPSpyFifoFull), 
    .TPSpyFIFO1_out(TPSpyFIFO1_out),   .TPSpyFIFO2_out(TPSpyFIFO2_out),
    .TPSpyFIFO3_out(TPSpyFIFO3_out),//   .TPSpyFIFO4_out(TPSpyFIFO4_out),
    .TTC_test0(TTC_test0),       .TTC_test1(TTC_test1), 
    .TTC_test2(TTC_test2),       .TTC_test3(TTC_test3), 
    .TTC_test4(TTC_test4),       .TTC_test5(TTC_test5), 
    .TTC_test6(TTC_test6),       .TTC_test7(TTC_test7), 
    .TTC_test8(TTC_test8),       .TTC_test9(TTC_test9), 
    .TTC_testA(TTC_testA),       .TTC_testB(TTC_testB), 
    .TTC_testC(TTC_testC),       .TTC_testD(TTC_testD), 
    .TTC_testE(TTC_testE),       .TTC_testF(TTC_testF), 
    .BunchEr(BunchEr), .mux_out(mux_out),
	 .latch_bucket(latch_bucket), .count_bucket(count_bucket),
	 .HistogramsDone(HistogramsDone),.HistogramMode(HistogramMode),.HZero(hist_zero),
	 .hist_localbus_read(hist_localbus_read),.hist_localbus_bin(hist_localbus_bin),
	 .hist_localbus_select(hist_localbus_select),
	 .hist0_value(latch_h0),.hist1_value(latch_h1),.hist2_value(latch_h2),.hist3_value(latch_h3),
	 .histsum_value(latch_sum)
        );

 commander commander (  
  .clk(SCLK_buf),    .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(1'b0), .TTC_Soft_rst(1'b0),
  .TTC_Start(1'b0), .TTC_Stop(1'b0), 
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), 
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst), 
  .run_status(run_status)       );

 always @ (posedge RxClk_Buf) begin 
   if (Soft_rst) begin
      RxClkCnt <= 0;  RxBC0Cnt <= 0;
   end
   else if (run_status) begin
      RxClkCnt <= RxClkCnt + 1;
      if (RxBC0) RxBC0Cnt <= RxBC0Cnt + 1;
   end
 end // always

//
// let's make sure we have the right number of buckets between "bc0"
//
reg[1:0] lhc_state;
parameter[1:0] BC0_WAIT=0, COUNT=1, WAIT_RESET=2;
 always @ (posedge TTC_CK180)
   if (~resetn) begin
	  lhc_state <= BC0_WAIT;
	  count_bucket <= 0; 
	  latch_bucket <= 0;
	end
   else case(lhc_state)
		BC0_WAIT: begin
			count_bucket <= 0;
			latch_bucket <= 0;
			if (run_status && bc0_strobe) lhc_state <= COUNT;
			else lhc_state <= BC0_WAIT;
			end
	   COUNT: begin
			if (bc0_strobe) begin
				latch_bucket <= bucket;
				lhc_state <= WAIT_RESET;
				end
			else begin
		      count_bucket <= count_bucket + 1;
			   lhc_state <= COUNT;
				end
			end
	   WAIT_RESET:
		   if (run_status) lhc_state <= WAIT_RESET;
			else lhc_state <= BC0_WAIT;
   endcase

//
// ethernet stuff, written by Jeremy Mans Nov 2005
//
// firstOrbit:  go in the header, it's an orbit number
// lastOrbit:   same as above...
// DestMAC:     register, Localbusslave
// HLXMAC:				"
// hlxId:            " 
// packet_len:  byte length of the payload (1024,1024,1024, then last histogram)  this is 1 at a time
//              so i'll have to calculate this later
// packet_send: signal to send it
// packet_sel:  which of 7 packets for the histogram (set to 0-6 for each packet)
// histogram_output_read: 32 bits of data, so it's the data in 2 successive buckets
// histogram_output_addr: 16 bits:  each address is a 32bit word.  bit 0 is always 0,
//                                  bit 8:1 is the address (0-255), and bits 14:9 is
//                                  the packet number (5:0 bits).  
//                                  
//
// here is the poop on what is going on.  each packet has 1024 bytes plus a header which is
// probably around 20-30 bytes.  we have to map these packets into histogram block ram.  since
// each histogram consists of 4 BRAMs, and each BRAM has 1024 words of 16 bits, then we will 
// need to use 2 packets per 1 BRAM.  since the total number of histogram bins is 3564, which
// corresponds to 7128 bytes, we only really need 7 packets (1024*7=7168>7128).  the first 6
// pacekts will have a length of 512 bins, the last (6) will have a length of 492 (6*512+492=3564).
// 
// packet numbers will go from: 0-6  [0-6]  for histogram 0 
//                              8-14 [8-E]   "      "     1
//                             16-22 [10-16] "      "     2
//                             24-30 [18-1E] "      "     3
//                             32-38 [20-26] "      "     4
//
// ==============================================================================================
// NOTES (Nov 2005) 
//   1. we will just go from 0-4 and use a separate VME register for histogram number (localbus
//      address 'h70, ethHistSel)
//   2. use localbus address 'h71 to read the packet number that would be sent next (packet_next)
//   3. use WriteOnlyCntrReg ('h61) bit 7 to reset ethernet packet pointer to 0 (ethPacketReset)
//   4. use         "        ('h61)  "  8 to send next packet (ethTransmit)
//   5. use Status Reg ('h10) bit 5 to indicate that the packet has been sent and the packet
//      number incremented (ethPacketSent)
//

always @ (posedge histogram_read_ack or negedge doingEthernet)
	if (~doingEthernet) ethCounter <= 0;
	else ethCounter <= ethCounter + 1;
//
// the easiest way to do this is to start the transmission, then wait
// about 90usec (86us on the LA...).  so to make it even easier, i'll
// start a 12-bit counter and increment with a 40MHz clock and when it
// gets to FFF then I can be pretty sure that the ethernet has finished.
//
reg[11:0] count_for_ethernet;
always @ (posedge TTC_CK)
  if (~resetn) count_for_ethernet <= 0;
  else count_for_ethernet <= doingEthernet ? count_for_ethernet + 1 : 0;

//assign alldone = ( (ethCounter+1) == packet_len - 1);
assign alldone = ( count_for_ethernet == 'hFFF );
wire ethBusy = xmit_busy;
parameter [1:0] eWait=0, eStart=1, eContinue=2, eDone=3;
always @ (posedge TTC_CK)
	if (~resetn) begin
		eState <= eWait;
		packet_next <= 0;
		packet_send <= 0;
		packet_len <= 0;
		doingEthernet <= 0;
		ethPacketSent <= 0;
	end
	else case (eState)
		eWait: begin
			doingEthernet <= 0;
			packet_send <= 0;
			packet_sel <= packet_next;
			packet_next <= ethPacketReset ? 'h0 : packet_next;
			packet_len <= ethLenOver ? ethLength : ( (packet_next=='h6) ? 'd984 : 'd1024 );
			if (ethTransmit) eState <= eStart;
			else eState <= eWait;
			end
		eStart: begin
		   //
			// packet_send is the signal to Jeremy's ethernet code.  we wait
			// until xmit_busy is asserted to be able to drop packet_send.
			//
			doingEthernet <= 1;
			ethPacketSent <= 0;
			packet_send <= 1;
			if (ethBusy) eState <= eContinue;
			else eState <= eStart;
			end
	   eContinue: begin
		   packet_send <= 0;
//			if (ethBusy) eState <= eContinue;
//			else eState <= eDone;
			if (alldone) eState <= eDone;
			else eState <= eContinue;
			end
		eDone: begin
			packet_next <= packet_next + 1;
			nbsent <= ethCounter;
			ethPacketSent <= 1;
			eState <= eWait;
			end
	endcase

assign histogram_output_read = ethData;
assign histogram_read_ack = doingEthernet & goRead;
assign ethAddr = histogram_output_addr[11:2];
assign ethRAM = histogram_output_addr[1];

transmitter_bundle theTransmitter(.clk100(SCLK_buf),.clk40(TTC_CK180),.PH_CLK(PH_CLK),
	.PH_RESET(PH_RESET), .PH_PAUSE(PH_PAUSE), .PH_SLEEP(PH_SLEEP), .PH_PDWN(PH_PDWN), .PH_SLEW(PH_SLEW),
	.PH_MDC(PH_MDC),.PH_MDIS(PH_MDIS),.PH_MDIO(PH_MDIO),
	.PH_RxD(PH_RxD),.PH_RxDV(PH_RxDV),.PH_RxER(PH_RxER),.PH_RxCLK(PH_RxCLK),.PH_TxD(PH_TxD),.PH_TxER(PH_TxER),
	.PH_TxEN(PH_TxEN),.PH_TxCLK(PH_TxCLK),.PH_COL(PH_COL),.PH_CRS(PH_CRS),
//	.firstOrbit(firstOrbit),.lastOrbit(lastOrbit),
	.firstOrbit(32'h0),.lastOrbit(32'd1023),
	.DestMAC(DestMAC),.HLXMAC(HLXMAC),.hlxId(hlxId),.xmit_busy(xmit_busy),
	// controllable inputs
	.packet_sel(packet_sel),                           // packet number (0-6)
	.packet_len(packet_len),                           // packet length (1024 for 0-5, 984 for packet 6)
	.packet_send(packet_send),									// module enable
	.histogram_output_read(histogram_output_read),	   // 32 bits of data
	.histogram_read_ack(histogram_read_ack),           // this tells the module that a new word is ready
	// controllable outputs
	.histogram_output_addr(histogram_output_addr),		// this is the address that i decode here
	.histogram_read_strobe(histogram_read_strobe),		// this says give me a new word
	.slave_state(slave_state),.global_enable(doingEthernet)
	);

endmodule