/*
Numbering in descriptor memory
400 -> Packet 0
404 -> Packet 0 (base address)
408 -> Packet 1
40C -> Packet 1 (base address)

*/
module mac_controller(mac_clk,ctl_addr,ctl_do,ctl_di,ctl_cycle,ctl_strobe,ctl_we,
                         ctl_ack,reset_phy,reset_mac,
								 packet_number, packet_length, send,
								 busy, OwnMAC);
    input mac_clk;
    output [11:2] ctl_addr;
    output [31:0] ctl_do;
    input [31:0] ctl_di;
    output ctl_cycle;
    output ctl_strobe, ctl_we;
    input ctl_ack;
    output reset_phy;
    output reset_mac;
    output busy;
	 input [5:0] packet_number;
	 input [10:0] packet_length;
	 input send;
	 input [47:0] OwnMAC;


	reg [11:0] poweron;

always @(posedge mac_clk) 
	if (poweron!=12'hFFF) poweron<=poweron+1;

	wire [31:0] init_do, sender_do;
	wire [11:2] init_addr, sender_addr;
	wire init_cycle, init_strobe, init_active;
	wire sender_cycle, sender_strobe, sender_active;

assign ctl_do=(init_active)?(init_do):((sender_active)?(sender_do):(32'h0));
assign ctl_addr=(init_active)?(init_addr):((sender_active)?(sender_addr):(10'h0));
assign ctl_cycle=(init_active)?(init_cycle):((sender_active)?(sender_cycle):(0));
assign ctl_strobe=(init_active)?(init_strobe):((sender_active)?(sender_strobe):(0));

assign busy=init_active|sender_active;

assign ctl_we=busy;

mac_control_init initManager(.sysclk(mac_clk),.active(init_active),
   .start(poweron==12'hFF0),
	.reset_phy(reset_phy),.reset_wb(reset_mac),
	.data_out(init_do),.cycle(init_cycle),.strobe(init_strobe),.ack(ctl_ack),.addr(init_addr),
	.MYMAC(OwnMAC));	


mac_control_send sendManager(.sysclk(mac_clk),.active(sender_active),.number(packet_number),.length(packet_length),.start(send),
	.data(sender_do),.cycle(sender_cycle),.strobe(sender_strobe),.ack(ctl_ack),.addr(sender_addr));
	

endmodule
