`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:    14:31:49 11/02/05
// Design Name:    
// Module Name:    transmitter_bundle
// Project Name:   
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module transmitter_bundle(clk100, clk40, PH_CLK,
	PH_RESET, PH_PAUSE, PH_SLEEP, PH_PDWN, PH_SLEW, PH_MDIO,
	PH_RxD,PH_RxDV,
                       PH_RxER,PH_RxCLK,PH_TxD,PH_TxER,PH_TxEN,PH_TxCLK,
                       PH_COL,PH_CRS,PH_MDC,PH_MDIS,
							  HLXMAC,DestMAC, packet_sel, packet_len, packet_send,
							  firstOrbit, lastOrbit, hlxId,
	 histogram_output_addr, histogram_output_read,
	 histogram_read_strobe, histogram_read_ack,
	 slave_state,global_enable,
	 xmit_busy
	);
    input clk100;
    input clk40;
	 output  PH_CLK, PH_RESET, PH_PAUSE, PH_SLEEP, PH_PDWN;
	 output [1:0] PH_SLEW;
	 inout PH_MDIO;
	 input [3:0] PH_RxD;
    input PH_RxDV;
    input PH_RxER;
    input PH_RxCLK;
    output [3:0] PH_TxD;
    output PH_TxER;
	 output PH_MDC, PH_MDIS;
    output PH_TxEN;
    input PH_TxCLK;
    input PH_COL;
    input PH_CRS;
	 wire Clk25, packet_clk;
	 input [47:0] HLXMAC,DestMAC;
	 input [5:0] packet_sel;
	 input [10:0] packet_len;
	 input packet_send;
	 input [31:0] firstOrbit, lastOrbit;
	 input [7:0] hlxId;
	 output [15:0] histogram_output_addr;
	 input [31:0] histogram_output_read;
	 output histogram_read_strobe,xmit_busy;
	 input histogram_read_ack;
	 output [2:0] slave_state;
	 input global_enable;
	 wire phy_reset_pos;

assign PH_MDIS = 1'h1; // disable MDIO
// Sets constants for the PHY (phy-specific behavior)
intel_lxt971a phy_manager(.reset(phy_reset_pos),.iReset_b(PH_RESET), .iPause(PH_PAUSE), .iSleep(PH_SLEEP), .iPowerDown(PH_PDWN),
	.iSlew(PH_SLEW));

assign packet_clk=clk40;


// MAC
	wire mac_reset, mac_clk;
	wire mdio_o, mdio_oe;
	wire mac_buf_we, mac_buf_cyc, mac_buf_stb, mac_buf_ack;
	wire [31:0] mac_buf_di, mac_buf_do, mac_buf_addr;
	wire mac_ctl_we, mac_ctl_cycle, mac_ctl_strobe, mac_ctl_ack;
	wire [31:0] mac_ctl_di, mac_ctl_do;
	wire [11:2] mac_ctl_addr;

assign PH_MDIO=(mdio_oe)?(mdio_o):(1'hz);
assign mac_clk=Clk25;

eth_top theMAC( .wb_rst_i(mac_reset),.wb_clk_i(mac_clk), 
	.mtx_clk_pad_i(PH_TxCLK), .mtxd_pad_o(PH_TxD), .mtxen_pad_o(PH_TxEN), .mtxerr_pad_o(PH_TxER),
	.mrx_clk_pad_i(PH_RxCLK), .mrxd_pad_i(PH_RxD), .mrxdv_pad_i(PH_RxDV), .mrxerr_pad_i(PH_RxER), .mcoll_pad_i(PH_COL), .mcrs_pad_i(PH_CRS),
	.mdc_pad_o(PH_MDC),.md_pad_i(PH_MDIO), .md_pad_o(mdio_o), .md_padoe_o(mdio_oe),
   .m_wb_adr_o(mac_buf_addr), .m_wb_we_o(mac_buf_we), 
   .m_wb_dat_o(mac_buf_do), .m_wb_dat_i(mac_buf_di), .m_wb_cyc_o(mac_buf_cyc), 
   .m_wb_stb_o(mac_buf_stb), .m_wb_ack_i(mac_buf_ack), .m_wb_err_i(1'h0),
   .wb_dat_i(mac_ctl_di), .wb_dat_o(mac_ctl_do), .wb_adr_i(mac_ctl_addr), .wb_sel_i(4'hf), .wb_we_i(mac_ctl_we), .wb_cyc_i(mac_ctl_cycle), .wb_stb_i(mac_ctl_strobe), .wb_ack_o(mac_ctl_ack)
);

	wire pkt_we, pkt_header_data;
	wire [15:0] pkt_read, pkt_write;
	wire [8:0] pkt_addr;
	wire [5:0] pkt_sel;

	wire [10:0] real_packet_len;
	wire mac_send_start;
	wire store_packet_mapping;
//	wire macctl_busy, macctl_packet_busy, macctl_poll;

// packet buffer
ethernet_packet_buffer packet_buffer(.wb_clk(mac_clk), .wb_addr(mac_buf_addr), .wb_di(mac_buf_do),.wb_do(mac_buf_di),
	.wb_cycle(mac_buf_cyc), .wb_strobe(mac_buf_stb), .wb_ack(mac_buf_ack), .wb_we(mac_buf_we),
	.packet_clk(packet_clk),.packet_header_data(pkt_header_data),.packet_sel(pkt_sel),.packet_addr(pkt_addr),
	.packet_read(pkt_read),.packet_write(pkt_write),.packet_we(pkt_we),
	.histogram_data(histogram_output_read),.histogram_addr(histogram_output_addr),
	.histogram_read_strobe(histogram_read_strobe),.histogram_read_ack(histogram_read_ack),
	.packet_store_mapping(store_packet_mapping),
	.slave_state(slave_state),.global_enable(global_enable)
//	.packet_send_poll(macctl_poll),.packet_send_busy(macctl_busy),.packet_send_macbusy()
	);

// mac controller
mac_controller mac_ctl(.mac_clk(mac_clk),.ctl_addr(mac_ctl_addr),.ctl_do(mac_ctl_di),.ctl_di(mac_ctl_do),
	.ctl_cycle(mac_ctl_cycle),.ctl_strobe(mac_ctl_strobe),.ctl_ack(mac_ctl_ack),.ctl_we(mac_ctl_we),
	.reset_phy(phy_reset_pos),.reset_mac(mac_reset), .OwnMAC(HLXMAC),
	.packet_number(pkt_sel),.packet_length(real_packet_len),.send(mac_send_start)
	);

	wire pacman_reset;
assign pacman_reset=mac_reset||phy_reset_pos;

// packet manager
packet_manager pacman(.clk(packet_clk),.reset(pacman_reset),.OwnMAC(HLXMAC),.DestMAC(DestMAC),
	.startOrbit(firstOrbit),.finishOrbit(lastOrbit),.hlxId(hlxId),
	.packet_send(packet_send),.packet_len(packet_len),.packet_sel(packet_sel),
	.packet_real_sel(pkt_sel),.packet_real_read(pkt_read),.packet_real_write(pkt_write),.packet_real_addr(pkt_addr),
	.packet_real_we(pkt_we), .packet_real_hd(pkt_header_data),
	.packet_real_send(mac_send_start), .packet_real_len(real_packet_len),
	.busy(xmit_busy),.store_packet_mapping(store_packet_mapping)); 



 wire   SCLK_buf=clk100;
                  
// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
  FDS flop1 (.D(1'b0), .C(SCLK_buf), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SCLK_buf), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SCLK_buf), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SCLK_buf), .Q(out4)); 
  assign ConfigRst = (out2 | out3 | out4); 


   DCM #(
      .CLKDV_DIVIDE(2.0), // Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
                          //   7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
      .CLKFX_DIVIDE(16),    // Can be any interger from 1 to 32
      .CLKFX_MULTIPLY(5),  // Can be any integer from 2 to 32
      .CLKIN_DIVIDE_BY_2("FALSE"), // TRUE/FALSE to enable CLKIN divide by two feature
      .CLKIN_PERIOD(80.0),  // Specify period of input clock
      .CLKOUT_PHASE_SHIFT("NONE"), // Specify phase shift of NONE, FIXED or VARIABLE
      .CLK_FEEDBACK("1X"),  // Specify clock feedback of NONE, 1X or 2X
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"), // SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
                                            //   an integer from 0 to 15
      .DFS_FREQUENCY_MODE("LOW"),  // HIGH or LOW frequency mode for frequency synthesis
      .DLL_FREQUENCY_MODE("LOW"),  // HIGH or LOW frequency mode for DLL
      .DUTY_CYCLE_CORRECTION("TRUE"), // Duty cycle correction, TRUE or FALSE
      .FACTORY_JF(16'hC080),   // FACTORY JF values
      .PHASE_SHIFT(0),     // Amount of fixed phase shift from -255 to 255
      .STARTUP_WAIT("FALSE")   // Delay configuration DONE until DCM LOCK, TRUE/FALSE
   ) 
   XTAL_DLL (
      .CLK0(XClk0),     // 0 degree DCM CLK ouptput
      .CLK180(), // 180 degree DCM CLK output
      .CLK270(), // 270 degree DCM CLK output
      .CLK2X(),   // 2X DCM CLK output
      .CLK2X180(), // 2X, 180 degree DCM CLK out
      .CLK90(),   // 90 degree DCM CLK output
      .CLKDV(),   // Divided DCM CLK out (CLKDV_DIVIDE)
      .CLKFX(ClkFx25),   // DCM CLK synthesis out (M/D)
      .CLKFX180(), // 180 degree CLK synthesis out
      .LOCKED(), // DCM LOCK status output
      .PSDONE(), // Dynamic phase adjust done output
      .STATUS(), // 8-bit DCM status bits output
      .CLKFB(XClk),   // DCM clock feedback
      .CLKIN(SCLK_buf),   // Clock input (from IBUFG, BUFG or DCM)
      .PSCLK(),   // Dynamic phase adjust clock input
      .PSEN(),     // Dynamic phase adjust enable input
      .PSINCDEC(), // Dynamic phase adjust increment/decrement
      .RST(ConfigRst)        // DCM asynchronous reset input
   );

  BUFG XClkBUF (.I(XClk0), .O(XClk));  
  BUFG Clk25BUF (.I(ClkFx25), .O(Clk25));  

assign PH_CLK=Clk25;

endmodule
